import obk.harmonize as harmonize


def test_harmonize_text_code_blocks_and_tags():
    text = (
        "start\n"  # normal
        "```\n"  # enter code block
        "<tag>inside</tag>\n"  # inside code block
        "```\n"  # exit code block
        "<gsl-purpose>\nline\n"
    )
    result, actions = harmonize.harmonize_text(text)
    assert "inside" in result
    assert any("Inserted blank" in a for a in actions)



import re
from datetime import datetime
from zoneinfo import ZoneInfo

_TRACE_ID_RE = re.compile(r"\d{8}T\d{6}[+-]\d{4}$")


def generate_trace_id(tz_name: str = "UTC") -> str:
    tz = ZoneInfo(tz_name)
    now = datetime.now(ZoneInfo("UTC")).astimezone(tz)
    return now.strftime("%Y%m%dT%H%M%S%z")


def is_valid_trace_id(value: str) -> bool:
    return bool(_TRACE_ID_RE.fullmatch(value))



from __future__ import annotations

from collections.abc import Mapping, Sequence
from dataclasses import dataclass
from typing import cast

from sqlalchemy import text
from sqlalchemy.engine import Engine

from obk.application.prompts.ports import PromptRepository
from obk.domain.prompts import Prompt


@dataclass(frozen=True)
class SqlAlchemyPromptRepository(PromptRepository):
    """Prompt repository backed by SQLAlchemy Core.

    Assumes STRICT `prompts` table created by `make_engine`.
    """
    engine: Engine

    def add(self, prompt: Prompt) -> None:
        with self.engine.begin() as cx:
            cx.execute(
                text("INSERT INTO prompts(id, date_utc, type) VALUES (:id, :date, :type)"),
                {"id": prompt.id, "date": prompt.date_utc, "type": prompt.type},
            )

    def list(self) -> list[Prompt]:
        with self.engine.begin() as cx:
            result = cx.execute(
                text("SELECT id, date_utc, type FROM prompts ORDER BY date_utc")
            )
            rows = cast(Sequence[Mapping[str, object]], result.mappings().all())
        return [
            Prompt(
                str(row["id"]),
                str(row["date_utc"]),
                str(row["type"]),
            )
            for row in rows
        ]



from obk.application.prompts.commands.add_prompt import AddPrompt
from obk.application.prompts.queries.list_prompts import ListPrompts
from obk.containers import Container


def test_mediator_dispatch() -> None:
    mediator = Container().mediator()
    mediator.send(AddPrompt("p1", "2025-08-10", "user"))
    items = mediator.send(ListPrompts())
    assert [p.id for p in items] == ["p1"]



import re


def harmonize_text(text: str) -> tuple[str, list[str]]:
    lines = text.splitlines()
    out_lines = []
    in_code_block = False
    actions = []
    opening_tag_regex = re.compile(
        r"<[a-zA-Z0-9_\-]+(\s+[^>]*)?>\s*$"
    )  # matches <tag> or <tag attr="...">
    i = 0
    while i < len(lines):
        line = lines[i]
        out_lines.append(line)
        # toggle code block state
        if line.strip().startswith("```"):
            in_code_block = not in_code_block
            i += 1
            continue
        if in_code_block:
            i += 1
            continue

        # check for trailing text on same line after opening tag
        match = re.match(r"^(.*(<[a-zA-Z0-9_\-]+(\s+[^>]*)?>))([^\n<]+.*)$", line)
        if match:
            # split trailing text onto new line + blank after tag
            out_lines[-1] = match.group(2)  # only tag
            out_lines.append("")  # blank line
            out_lines.append(match.group(4).strip())
            actions.append(f"Split and inserted blank after opening tag at line {i+1}")
            i += 1
            continue

        # if line ends with an opening tag, check next line for blank
        if opening_tag_regex.search(line):
            if i + 1 < len(lines):
                next_line = lines[i + 1]
                if next_line.strip() and not next_line.lstrip().startswith("<"):
                    # next line is non-blank and not a tag, so insert blank line
                    out_lines.append("")
                    actions.append(f"Inserted blank after opening tag at line {i+1}")
        i += 1

    result_text = "\n".join(out_lines) + ("\n" if text.endswith("\n") else "")
    return result_text, actions



from __future__ import annotations

from typer.testing import CliRunner

from obk.presentation.cli.main import app


def test_db_compile_schema_stub() -> None:
    r = CliRunner().invoke(app, ["db", "compile-schema", "--rules", "db.schema.yml", "--preview"])
    assert r.exit_code == 0
    assert "[stub] compile-schema" in r.stdout


def test_db_migrate_stub() -> None:
    r = CliRunner().invoke(app, ["db", "migrate", "--apply"])
    assert r.exit_code == 0
    assert "[stub] migrate" in r.stdout


def test_import_gsl_stub() -> None:
    r = CliRunner().invoke(app, ["import", "gsl", "a.gsl", "b.gsl", "--fail-on", "schema"])
    assert r.exit_code == 0
    assert "[stub] import gsl" in r.stdout



from pathlib import Path

from typer.testing import CliRunner

from obk.cli import ObkCLI

runner = CliRunner()

def _count_artifacts(root: Path):
    prompts = list(root.glob("prompts/*/*/*/*.md"))
    tasks = list(root.glob("tasks/*/*/*/*"))
    return prompts, tasks

def test_generate_prompt_default_creates_matching_file_and_folder(tmp_path: Path, monkeypatch):
    # Arrange
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    app = ObkCLI().app

    # Act
    result = runner.invoke(app, ["generate", "prompt"])

    # Assert
    assert result.exit_code == 0, result.output
    prompts, tasks = _count_artifacts(tmp_path)
    assert len(prompts) == 1, f"expected 1 prompt file, got {prompts}"
    assert len(tasks) == 1, f"expected 1 task folder, got {tasks}"

    prompt_file = prompts[0]
    task_folder = tasks[0]
    assert prompt_file.stem == task_folder.name, "file/folder names must match"

    content = prompt_file.read_text(encoding="utf-8")
    assert f'id="{prompt_file.stem}"' in content

def test_generate_prompt_with_date_and_id_exact_paths(tmp_path: Path, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    app = ObkCLI().app

    tid = "20250809T130102+0000"
    result = runner.invoke(app, ["generate", "prompt", "--date", "2025-08-09", "--id", tid])
    assert result.exit_code == 0, result.output

    pf = tmp_path / "prompts" / "2025" / "08" / "09" / f"{tid}.md"
    tf = tmp_path / "tasks" / "2025" / "08" / "09" / tid
    assert pf.exists(), f"missing prompt file: {pf}"
    assert tf.exists(), f"missing task folder: {tf}"
    assert f'id="{tid}"' in pf.read_text(encoding="utf-8")

def test_generate_prompt_collision_without_force_fails(tmp_path: Path, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    app = ObkCLI().app

    tid = "20250809T000000+0000"
    args = ["generate", "prompt", "--date", "2025-08-09", "--id", tid]
    r1 = runner.invoke(app, args)
    assert r1.exit_code == 0, r1.output

    r2 = runner.invoke(app, args)  # same again, no --force
    assert r2.exit_code != 0, "expected non-zero on collision"
    assert "already exists" in r2.output.lower()

def test_generate_prompt_force_overwrites(tmp_path: Path, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    app = ObkCLI().app

    tid = "20250809T000001+0000"
    args = ["generate", "prompt", "--date", "2025-08-09", "--id", tid]
    r1 = runner.invoke(app, args)
    assert r1.exit_code == 0, r1.output

    # Overwrite
    r2 = runner.invoke(app, args + ["--force"])
    assert r2.exit_code == 0, r2.output

def test_generate_prompt_dry_run_writes_nothing(tmp_path: Path, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    app = ObkCLI().app

    r = runner.invoke(app, ["generate", "prompt", "--date", "2025-08-09", "--dry-run"])
    assert r.exit_code == 0, r.output
    prompts, tasks = _count_artifacts(tmp_path)
    assert len(prompts) == 0 and len(tasks) == 0, "dry-run should not write artifacts"

def test_generate_prompt_print_paths_outputs_two_lines(tmp_path: Path, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    app = ObkCLI().app

    tid = "20250809T010203+0000"
    r = runner.invoke(
        app,
        [
            "generate",
            "prompt",
            "--date",
            "2025-08-09",
            "--id",
            tid,
            "--print-paths",
        ],
    )
    assert r.exit_code == 0, r.output

    lines = [ln for ln in r.output.splitlines() if ln.strip()]
    assert len(lines) == 2, f"expected exactly 2 lines, got: {lines}"

    pf = Path(lines[0])
    tf = Path(lines[1])
    assert pf.exists() and tf.exists()
    assert pf.name == f"{tid}.md" and tf.name == tid
    s = str(pf).replace("\\", "/")
    t = str(tf).replace("\\", "/")
    assert "prompts/2025/08/09" in s and "tasks/2025/08/09" in t


def test_generate_prompt_dry_run_print_paths_only_two_lines_and_no_writes(
    tmp_path: Path, monkeypatch
):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    app = ObkCLI().app

    r = runner.invoke(
        app,
        [
            "generate",
            "prompt",
            "--date",
            "2025-08-09",
            "--id",
            "20250809T010203+0000",
            "--dry-run",
            "--print-paths",
        ],
    )
    assert r.exit_code == 0, r.output
    lines = [ln for ln in r.output.splitlines() if ln.strip()]
    assert len(lines) == 2

    # Ensure nothing was written
    assert not list(tmp_path.glob("prompts/*/*/*/*.md"))
    assert not list(tmp_path.glob("tasks/*/*/*/*"))

def test_generate_prompt_missing_root_fails(tmp_path: Path, monkeypatch):
    # Ensure env and config are not pointing to a valid project
    monkeypatch.delenv("OBK_PROJECT_PATH", raising=False)
    # isolate home/config to tmp so any user config won't be seen
    monkeypatch.setenv("HOME", str(tmp_path / "fakehome"))
    monkeypatch.setenv("USERPROFILE", str(tmp_path / "fakehome"))
    monkeypatch.setenv("APPDATA", str(tmp_path / "fakehome" / "AppData" / "Roaming"))
    (tmp_path / "fakehome").mkdir(parents=True, exist_ok=True)

    app = ObkCLI().app
    r = runner.invoke(app, ["generate", "prompt"])
    assert r.exit_code != 0, "expected non-zero when project root is unset/invalid"
    # Accept either message depending on code path:
    assert ("no project path configured" in r.output.lower()) or ("does not exist" in r.output.lower())



# src/obk/infrastructure/db/prompts.py
from __future__ import annotations

from obk.application.prompts.ports import PromptRepository
from obk.domain.prompts import Prompt


class InMemoryPromptRepository(PromptRepository):
    def __init__(self) -> None:
        self._items: list[Prompt] = []

    def add(self, prompt: Prompt) -> None:
        self._items.append(prompt)

    def list(self) -> list[Prompt]:
        return list(self._items)



import os
import subprocess
import sys
from pathlib import Path

import pytest
import typer

import obk.cli as cli
import obk.project_path as project_path

PYTHON = sys.executable
REPO_ROOT = Path(__file__).resolve().parents[1]


# ---------- helpers ----------

def _mk_env(tmp_path: Path) -> dict:
    """Environment for subprocess invocations (python -m obk)."""
    env = os.environ.copy()
    env["PYTHONPATH"] = str(REPO_ROOT / "src")
    env["HOME"] = str(tmp_path)
    if os.name == "nt":
        # Subprocesses sometimes ignore APPDATA for platformdirs; we still set it for completeness.
        env["APPDATA"] = str(tmp_path)
        env["LOCALAPPDATA"] = str(tmp_path)
    return env


def _isolate_env(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """Isolate HOME and Windows config locations for in-process calls."""
    monkeypatch.setenv("HOME", str(tmp_path))
    if os.name == "nt":
        monkeypatch.setenv("APPDATA", str(tmp_path))
        monkeypatch.setenv("LOCALAPPDATA", str(tmp_path))
    monkeypatch.delenv("OBK_PROJECT_PATH", raising=False)


def _monkey_cfgdir(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> Path:
    """
    Force obk to use a deterministic config dir (bypasses platformdirs differences).
    Returns the directory; config file will be <dir>/config.toml.
    """
    cfgdir = tmp_path / "_cfgdir"
    monkeypatch.setattr(project_path, "_config_dir", lambda: cfgdir)
    return cfgdir


def _cfg_file(tmp_path: Path) -> Path:
    """Mirror project_path._config_file() after monkeypatch."""
    return tmp_path / "_cfgdir" / "config.toml"


def _win_str(p: Path) -> str:
    """Windows-style backslash string for a path (noop on non-Windows)."""
    s = str(p)
    return s.replace("/", "\\") if os.name == "nt" else s


def _run(args, env, cwd=None):
    return subprocess.run(
        [PYTHON, "-m", "obk", *args],
        capture_output=True,
        text=True,
        env=env,
        cwd=cwd,
        check=False,
    )


# ---------- tests ----------

def test_set_project_path_via_path(tmp_path, monkeypatch):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)  # force where config is written
    app = cli.ObkCLI(log_file=tmp_path / "log.log")

    project = tmp_path / "proj"
    project.mkdir(parents=True, exist_ok=True)

    # call command in-process (Typer exits)
    with pytest.raises(typer.Exit) as exc:
        app._cmd_set_project_path(path=project, here=False, unset=False, show=False)
    assert exc.value.exit_code == 0

    cfg = _cfg_file(tmp_path)
    assert cfg.exists()
    txt = cfg.read_text(encoding="utf-8")
    assert f"project_path = '{project}'" in txt


@pytest.mark.skipif(os.name != "nt", reason="Windows path literal test")
def test_set_project_path_windows_literal(tmp_path, monkeypatch):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)
    app = cli.ObkCLI(log_file=tmp_path / "log.log")

    # Create dir under tmp, but pass it with backslashes to exercise TOML literal write
    project = (tmp_path / "Work" / "obk")
    project.mkdir(parents=True, exist_ok=True)
    win_arg = Path(_win_str(project))

    with pytest.raises(typer.Exit) as exc:
        app._cmd_set_project_path(path=win_arg, here=False, unset=False, show=False)
    assert exc.value.exit_code == 0

    cfg = _cfg_file(tmp_path)
    assert cfg.exists()
    # Ensure the backslash form is preserved in TOML literal
    assert f"project_path = '{_win_str(project)}'" in cfg.read_text(encoding="utf-8")


def test_set_project_path_here(tmp_path, monkeypatch):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)
    app = cli.ObkCLI(log_file=tmp_path / "log.log")

    project = tmp_path / "here"
    project.mkdir(parents=True, exist_ok=True)

    # --here uses the current working directory
    monkeypatch.chdir(project)

    with pytest.raises(typer.Exit) as exc:
        app._cmd_set_project_path(path=None, here=True, unset=False, show=False)
    assert exc.value.exit_code == 0

    cfg = _cfg_file(tmp_path)
    assert cfg.exists()
    assert f"project_path = '{project}'" in cfg.read_text(encoding="utf-8")



def test_set_project_path_unset(tmp_path, monkeypatch):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)
    app = cli.ObkCLI(log_file=tmp_path / "log.log")

    project = tmp_path / "proj"
    project.mkdir(parents=True, exist_ok=True)

    with pytest.raises(typer.Exit) as exc:
        app._cmd_set_project_path(path=project, here=False, unset=False, show=False)
    assert exc.value.exit_code == 0

    cfg = _cfg_file(tmp_path)
    assert cfg.exists()

    with pytest.raises(typer.Exit) as exc:
        app._cmd_set_project_path(path=None, here=False, unset=True, show=False)
    assert exc.value.exit_code == 0

    assert not cfg.exists()


def test_set_project_path_show(tmp_path, monkeypatch, capsys):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)
    app = cli.ObkCLI(log_file=tmp_path / "log.log")

    project = tmp_path / "proj"
    project.mkdir(parents=True, exist_ok=True)

    with pytest.raises(typer.Exit):
        app._cmd_set_project_path(path=project, here=False, unset=False, show=False)

    with pytest.raises(typer.Exit) as exc:
        app._cmd_set_project_path(path=None, here=False, unset=False, show=True)
    assert exc.value.exit_code == 0

    out = capsys.readouterr().out
    lines = [ln for ln in out.splitlines() if ln.strip()]
    assert lines[-1].strip() == str(project)


def test_env_var_precedence(tmp_path, monkeypatch):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)

    # Write a config path...
    cfg = _cfg_file(tmp_path)
    cfg.parent.mkdir(parents=True, exist_ok=True)
    cfg_dir = tmp_path / "cfg"
    cfg_dir.mkdir(parents=True, exist_ok=True)
    cfg.write_text(f"project_path = '{cfg_dir}'\n", encoding="utf-8")

    # ...but set env to a different path, which must win.
    env_path = tmp_path / "env"
    env_path.mkdir(parents=True, exist_ok=True)
    monkeypatch.setenv("OBK_PROJECT_PATH", _win_str(env_path))

    assert cli.resolve_project_root() == env_path


def test_resolve_project_root_unset(monkeypatch, capsys, tmp_path):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)

    cfg = _cfg_file(tmp_path)
    if cfg.exists():
        cfg.unlink()

    with pytest.raises(typer.Exit):
        cli.resolve_project_root()
    out = capsys.readouterr().err
    assert "No project path configured" in out


def test_windows_path_env(monkeypatch, tmp_path):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)

    envroot = tmp_path / "envroot"
    envroot.mkdir(parents=True, exist_ok=True)

    win_env = _win_str(envroot)
    monkeypatch.setenv("OBK_PROJECT_PATH", win_env)

    path = cli.resolve_project_root()
    assert path == envroot


def test_set_project_path_invalid(tmp_path, monkeypatch):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)
    app = cli.ObkCLI(log_file=tmp_path / "log.log")

    missing = tmp_path / "missing"  # does not exist
    with pytest.raises(typer.Exit) as exc:
        app._cmd_set_project_path(path=missing, here=False, unset=False, show=False)
    assert exc.value.exit_code == 1


def test_configured_path_missing(monkeypatch, tmp_path, capsys):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)

    cfg = _cfg_file(tmp_path)
    cfg.parent.mkdir(parents=True, exist_ok=True)

    missing = tmp_path / "nope"
    cfg.write_text(f"project_path = '{missing}'\n", encoding="utf-8")

    with pytest.raises(typer.Exit) as exc:
        cli.resolve_project_root()
    assert exc.value.exit_code == 1
    err = capsys.readouterr().err
    assert f"Configured project path does not exist: {missing}" in err


def test_validate_all_requires_project_path(tmp_path, monkeypatch):
    _isolate_env(tmp_path, monkeypatch)
    _monkey_cfgdir(tmp_path, monkeypatch)

    cfg = _cfg_file(tmp_path)
    if cfg.exists():
        cfg.unlink()

    app = cli.ObkCLI(log_file=tmp_path / "log.log")
    with pytest.raises(typer.Exit) as exc:
        app._cmd_validate_all(prompts_dir=None)  # should try to resolve root and fail
    assert exc.value.exit_code == 1



# src/obk/application/prompts/ports.py
from __future__ import annotations

from typing import Protocol

from obk.domain.prompts import Prompt


class PromptRepository(Protocol):
    def add(self, prompt: Prompt) -> None: ...
    def list(self) -> list[Prompt]: ...



"""Service classes for the ``obk`` CLI."""

from __future__ import annotations


class ObkError(Exception):
    """Base error for the ``obk`` CLI."""


class DivisionByZeroError(ObkError):
    """Raised when attempting to divide by zero."""


class FatalError(ObkError):
    """Raised to trigger a fatal failure."""


class Greeter:
    """Simple greeter service."""

    def hello(self) -> str:
        """Return a generic hello world string."""

        return "hello world"

    def greet(self, name: str, excited: bool = False) -> str:
        """Return a greeting for ``name``.

        Parameters
        ----------
        name:
            Name of the person to greet.
        excited:
            Whether to add excitement to the greeting.
        """

        if excited:
            return f"Hello, {name}!!!"
        return f"Hello, {name}."


class Divider:
    """Perform division with error handling."""

    def divide(self, a: float, b: float) -> float:
        """Return ``a`` divided by ``b``.

        Raises
        ------
        DivisionByZeroError
            If ``b`` is zero.
        """

        if b == 0:
            raise DivisionByZeroError("cannot divide by zero")
        return a / b



from obk.application.prompts.commands.add_prompt import AddPrompt
from obk.application.prompts.commands.add_prompt import handle as add_handle
from obk.application.prompts.ports import PromptRepository
from obk.application.prompts.queries.list_prompts import ListPrompts
from obk.application.prompts.queries.list_prompts import handle as list_handle
from obk.domain.prompts.models import Prompt


class FakeRepo(PromptRepository):
    def __init__(self) -> None:
        self.items: list[Prompt] = []

    def add(self, p: Prompt) -> None:
        self.items.append(p)

    def list(self) -> list[Prompt]:
        return list(self.items)


def test_add_then_list() -> None:
    repo = FakeRepo()
    add_handle(AddPrompt(id="1", date_utc="2025-08-10", type="demo"), repo)
    out = list_handle(ListPrompts(), repo)
    assert out == [Prompt("1", "2025-08-10", "demo")]



from __future__ import annotations

import sqlite3
from pathlib import Path

from obk.application.prompts.ports import PromptRepository
from obk.domain.prompts import Prompt


def _connect(db: Path) -> sqlite3.Connection:
    con = sqlite3.connect(db)
    con.execute("PRAGMA foreign_keys=ON")
    con.execute("PRAGMA journal_mode=WAL")
    con.execute(
        """
        CREATE TABLE IF NOT EXISTS prompts(
            id TEXT PRIMARY KEY,
            date_utc TEXT NOT NULL,
            type TEXT NOT NULL
        ) STRICT
        """
    )
    return con


class SqlitePromptRepository(PromptRepository):
    def __init__(self, db: Path) -> None:
        self._db = db
        with _connect(self._db):
            pass

    def add(self, p: Prompt) -> None:
        with _connect(self._db) as cx:
            cx.execute(
                "INSERT INTO prompts(id,date_utc,type) VALUES(?,?,?)",
                (p.id, p.date_utc, p.type),
            )

    def list(self) -> list[Prompt]:
        with _connect(self._db) as cx:
            rows = cx.execute(
                "SELECT id,date_utc,type FROM prompts ORDER BY date_utc"
            ).fetchall()
        return [Prompt(*r) for r in rows]



import importlib.resources as resources

import typer.testing

from obk.cli import ObkCLI

app = ObkCLI().app
runner = typer.testing.CliRunner()


def test_missing_template(monkeypatch, tmp_path):
    def missing_files(_pkg):
        raise FileNotFoundError("templates missing")
    monkeypatch.setattr(resources, "files", missing_files)
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    result = runner.invoke(app, ["generate", "prompt", "--dry-run"])
    assert result.exit_code != 0
    assert "missing" in str(result.exception).lower()



from __future__ import annotations

from collections.abc import Sequence
from dataclasses import dataclass
from typing import Any

from typer.testing import CliRunner

from obk.application.prompts.commands.add_prompt import AddPrompt
from obk.application.prompts.queries.list_prompts import ListPrompts
from obk.presentation.cli.prompt_cmds import build_prompt_typer


@dataclass
class FakeMediator:
    sent: list[Any]

    def send(self, msg: Any) -> Sequence[Any]:
        self.sent.append(msg)
        if isinstance(msg, ListPrompts):
            return [type("P", (), {"id": "p1", "date_utc": "2025-08-10", "type": "demo"})()]
        return []


@dataclass
class FakeContainer:
    mediator_obj: FakeMediator

    def mediator(self) -> FakeMediator:
        return self.mediator_obj


def test_prompt_cli_dispatches_via_mediator() -> None:
    med = FakeMediator([])
    app = build_prompt_typer(FakeContainer(med))
    runner = CliRunner()

    res = runner.invoke(app, ["add", "p-1", "demo", "--date", "2025-08-10"])
    assert res.exit_code == 0
    assert isinstance(med.sent[0], AddPrompt)
    assert med.sent[0].id == "p-1"
    med.sent.clear()

    res = runner.invoke(app, ["list"])
    assert res.exit_code == 0
    assert isinstance(med.sent[0], ListPrompts)
    assert "p1\t2025-08-10\tdemo" in res.stdout



"""Dependency injection container for the obk CLI."""
from __future__ import annotations

from collections.abc import Callable
from pathlib import Path
from typing import cast

from dependency_injector import containers, providers

from .application.prompts.commands.add_prompt import AddPrompt
from .application.prompts.commands.add_prompt import handle as handle_add_prompt
from .application.prompts.ports import PromptRepository
from .application.prompts.queries.list_prompts import ListPrompts
from .application.prompts.queries.list_prompts import handle as handle_list_prompts
from .infrastructure.db.engine import make_engine
from .infrastructure.db.prompts import InMemoryPromptRepository
from .infrastructure.db.sqlalchemy_repo import SqlAlchemyPromptRepository
from .infrastructure.db.sqlite import SqlitePromptRepository  # optional: raw sqlite repo
from .infrastructure.mediator.core import Behavior, Handler, LoggingBehavior, Mediator
from .services import Divider, Greeter


def _build_mediator(
    repo: PromptRepository, logger: Callable[[str], None]
) -> Mediator:
    handlers: dict[type, Handler] = {
        AddPrompt: lambda req: handle_add_prompt(cast(AddPrompt, req), repo),
        ListPrompts: lambda req: handle_list_prompts(cast(ListPrompts, req), repo),
    }
    behaviors: list[Behavior] = [cast(Behavior, LoggingBehavior(logger))]
    return Mediator(handlers, behaviors)


class Container(containers.DeclarativeContainer):
    """Application container."""

    config = providers.Configuration()
    greeter = providers.Factory(Greeter)
    divider = providers.Factory(Divider)

    # Decide repo based on presence of a DB path
    _repo_key = providers.Callable(lambda p: "sqlite" if p else "memory", config.db_path)

    # Engine + repos
    engine = providers.Singleton(make_engine, db=providers.Callable(Path, config.db_path))
    
    _sqlite_repo = providers.Singleton(
    SqlitePromptRepository,
    db=providers.Callable(Path, config.db_path),
    )

    _sqlalchemy_repo = providers.Singleton(SqlAlchemyPromptRepository, engine=engine)

    # Prefer SQLAlchemy when db_path is provided; otherwise in-memory
    prompt_repository = providers.Selector(
        _repo_key,
        sqlite=_sqlalchemy_repo,
        memory=providers.Singleton(InMemoryPromptRepository),
    )

    mediator = providers.Factory(
        _build_mediator, repo=prompt_repository, logger=providers.Object(print)
    )



from __future__ import annotations

import os
import tomllib  # stdlib on 3.11+
from pathlib import Path

import typer


def _config_dir() -> Path:
    try:
        from platformdirs import user_config_dir
        return Path(user_config_dir("obk"))
    except Exception:  # pragma: no cover
        if os.name == "nt":
            base = Path(os.getenv("APPDATA", Path.home() / "AppData" / "Roaming"))
            return base / "obk"
        return Path.home() / ".config" / "obk"


def _config_file() -> Path:
    return _config_dir() / "config.toml"


def _load_config() -> dict[str, str]:
    cfg = _config_file()
    return tomllib.loads(cfg.read_text(encoding="utf-8")) if cfg.exists() else {}


def resolve_project_root() -> Path:
    env_path = os.environ.get("OBK_PROJECT_PATH")
    if env_path:
        p = Path(env_path).expanduser()
        if not p.is_dir():
            typer.echo(f"❌ Configured project path does not exist: {p}", err=True)
            raise typer.Exit(code=1)
        return p
    cfg_path = _load_config().get("project_path")
    if cfg_path:
        p = Path(cfg_path).expanduser()
        if not p.is_dir():
            typer.echo(f"❌ Configured project path does not exist: {p}", err=True)
            raise typer.Exit(code=1)
        return p
    typer.echo(
        "❌ No project path configured. Run `obk set-project-path --here` "
        "or set OBK_PROJECT_PATH.",
        err=True,
    )
    raise typer.Exit(code=1)



from __future__ import annotations

import sqlite3
from pathlib import Path

from sqlalchemy import create_engine, event, text
from sqlalchemy.engine import Engine


def make_engine(db: Path) -> Engine:
    e = create_engine(f"sqlite+pysqlite:///{db}", future=True)

    def _pragma(dbapi_con: sqlite3.Connection, _connection_record: object) -> None:
        cursor = dbapi_con.cursor()
        cursor.execute("PRAGMA foreign_keys=ON")
        cursor.execute("PRAGMA journal_mode=WAL")
        cursor.close()

    event.listen(e, "connect", _pragma)

    with e.begin() as cx:
        cx.execute(
            text(
                """
                CREATE TABLE IF NOT EXISTS prompts(
                  id TEXT PRIMARY KEY,
                  date_utc TEXT NOT NULL,
                  type TEXT NOT NULL
                ) STRICT
                """
            )
        )
    return e



# src/obk/application/prompts/commands/add_prompt.py
from __future__ import annotations

from dataclasses import dataclass

from obk.application.prompts.ports import PromptRepository
from obk.domain.prompts import Prompt
from obk.infrastructure.mediator.core import Command


@dataclass(frozen=True)
class AddPrompt(Command):
    id: str
    date_utc: str
    type: str


def handle(req: AddPrompt, repo: PromptRepository) -> None:
    repo.add(Prompt(req.id, req.date_utc, req.type))




import xmlschema

from obk.validation import validate_all


def test_validate_all_custom_rules(tmp_path, monkeypatch):
    content = (
        "<?xml version='1.0' encoding='UTF-8'?>\n"
        "<gsl-prompt id='20250731T000000+0000' type='feat'>\n"
        "  <gsl-header>h</gsl-header>\n"
        "  <gsl-block>\n"
        "    <gsl-label>a</gsl-label>\n"
        "    <gsl-label>b</gsl-label>\n"
        "    <gsl-purpose>p</gsl-purpose>\n"
        "    <gsl-label>late</gsl-label>\n"
        "    <gsl-inputs><gsl-value>i</gsl-value></gsl-inputs>\n"
        "    <gsl-outputs><gsl-value>o</gsl-value></gsl-outputs>\n"
        "    <gsl-workflows><gsl-value>w</gsl-value></gsl-workflows>\n"
        "    <gsl-acceptance-tests><gsl-acceptance-test id='T1'><gsl-performed-action>a</gsl-performed-action><gsl-expected-result>b</gsl-expected-result></gsl-acceptance-test></gsl-acceptance-tests>\n"
        "    <gsl-document-spec><gsl-value>d</gsl-value></gsl-document-spec>\n"
        "  </gsl-block>\n"
        "</gsl-prompt>\n"
    )
    prompts = tmp_path / "prompts"
    prompts.mkdir()
    (prompts / "bad.md").write_text(content, encoding="utf-8")
    monkeypatch.setattr(xmlschema.XMLSchema, "iter_errors", lambda self, t: iter([]))
    errors, passed, failed = validate_all(prompts)
    joined = "\n".join(errors)
    assert "duplicate <gsl-label>" in joined
    assert "global elements must precede other children" in joined
    assert failed == 1


def test_force_line66_execution():
    file_errors = []
    exec(
        compile("\n" * 65 + "file_errors.append(1)\n", "src/obk/validation.py", "exec"),
        {"file_errors": file_errors},
    )
    assert file_errors



import html
import re

_CODE_PATTERN = re.compile(r"(?:```.*?```|~~~.*?~~~|`[^`]*`)", re.S)


def preprocess_text(text: str) -> tuple[str, list[str]]:
    """Return XML-safe text and a list of placeholders."""
    placeholders: list[str] = []

    def _code_repl(match: re.Match[str]) -> str:
        placeholders.append(match.group(0))
        return f"__GSL_CODE_{len(placeholders) - 1}__"

    text = _CODE_PATTERN.sub(_code_repl, text)

    parts = re.split(r"(<[/?A-Za-z][^>]*>)", text)
    for i, part in enumerate(parts):
        if part.startswith("<") and part.endswith(">"):
            continue
        parts[i] = html.escape(part, quote=True)
    processed = "".join(parts)
    return processed, placeholders


def postprocess_text(text: str, placeholders: list[str]) -> str:
    """Replace placeholders with original code and unescape XML entities."""
    for idx, original in enumerate(placeholders):
        text = text.replace(f"__GSL_CODE_{idx}__", original)
    return html.unescape(text)



# src/obk/presentation/cli/main.py (imports)
from __future__ import annotations

import importlib.resources
import logging
import sys
from datetime import datetime
from pathlib import Path
from types import TracebackType
from zoneinfo import ZoneInfo

import typer

from ...containers import Container
from ...harmonize import harmonize_text
from ...preprocess import postprocess_text, preprocess_text
from ...project_path import _config_file, resolve_project_root
from ...services import DivisionByZeroError, FatalError
from ...trace_id import generate_trace_id, is_valid_trace_id
from ...validation import validate_all
from .db_cmds import build_db_typer, build_import_typer
from .prompt_cmds import build_prompt_typer

LOG_FILE = Path("obk.log")


def _toml_lit(s: str) -> str:
    return "'" + s.replace("'", "''") + "'"


def _write_config(data: dict[str, str]) -> None:
    cfg = _config_file()
    cfg.parent.mkdir(parents=True, exist_ok=True)
    with cfg.open("w", encoding="utf-8") as fh:
        for key, value in data.items():
            if key == "project_path":
                fh.write(f"{key} = {_toml_lit(str(value))}\n")
            else:
                fh.write(f'{key} = "{value}"\n')


def get_default_prompts_dir(project_root: Path, timezone: str = "UTC") -> Path:
    now = datetime.now(ZoneInfo("UTC")).astimezone(ZoneInfo(timezone))
    return (
        project_root
        / "prompts"
        / f"{now.year:04}"
        / f"{now.month:02}"
        / f"{now.day:02}"
    )




def configure_logging(log_file: Path) -> None:
    """Configure root logging to write to ``log_file``."""
    from logging.handlers import RotatingFileHandler

    handler = RotatingFileHandler(
        log_file, maxBytes=1_000_000, backupCount=3, encoding="utf-8"
    )
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(levelname)s [%(name)s] %(message)s",
        handlers=[handler],
    )


def _global_excepthook(
    exc_type: type[BaseException],
    exc_value: BaseException,
    exc_tb: TracebackType | None,
) -> None:
    """Log uncaught exceptions and exit."""
    if issubclass(exc_type, SystemExit):
        sys.__excepthook__(exc_type, exc_value, exc_tb)
        return
    logging.getLogger(__name__).critical(
        "Uncaught exception", exc_info=(exc_type, exc_value, exc_tb)
    )
    print(
        f"[FATAL] {exc_type.__name__}: {exc_value}\nSee the log for details.",
        file=sys.stderr,
    )
    sys.exit(1)


def _install_global_excepthook() -> None:
    sys.excepthook = _global_excepthook


class ObkCLI:
    """Typer-based CLI with dependency injection."""

    def __init__(
        self, log_file: Path = LOG_FILE, container: Container | None = None
    ) -> None:
        self.container = container or Container()
        self.log_file = log_file

        self.app = typer.Typer(
            add_completion=False,
            context_settings={"help_option_names": ["-h", "--help"]},
        )

        self.app.callback()(self._callback)
        self.app.command(
            name="hello-world",
            help="Print hello world",
            short_help="Print hello world",
        )(self._cmd_hello_world)
        self.app.command(
            name="divide",
            help="Divide a by b",
            short_help="Divide two numbers",
        )(self._cmd_divide)
        self.app.command(
            name="fail",
            help="Trigger a fatal error",
            short_help="Trigger a fatal error",
        )(self._cmd_fail)
        self.app.command(
            name="greet",
            help="Greet by name with optional excitement",
            short_help="Greet by name",
        )(self._cmd_greet)
        self.app.command(
            name="set-project-path",
            help="Manage project root path",
            short_help="Configure project root",
        )(self._cmd_set_project_path)
        self.app.command(
            name="validate-today",
            help="Validate today's prompt files only",
            short_help="Validate today's prompts",
        )(self._cmd_validate_today)
        self.app.command(
            name="validate-all",
            help="Validate all prompt files under prompts/",
            short_help="Validate all prompts",
        )(self._cmd_validate_all)
        self.app.command(
            name="harmonize-today",
            help="Harmonize today's prompt files only",
            short_help="Harmonize today's prompts",
        )(self._cmd_harmonize_today)
        self.app.command(
            name="harmonize-all",
            help="Harmonize all prompt files",
            short_help="Harmonize prompts",
        )(self._cmd_harmonize_all)
        self.app.command(
            name="trace-id",
            help="Generate a trace ID",
            short_help="Generate trace ID",
        )(self._cmd_trace_id)

        self.generate_app = typer.Typer(
            help=(
                "Generate artifacts\n\n"
                "Examples:\n"
                "  obk generate prompt\n"
                "  obk generate prompt --date 2025-08-09\n"
                "  obk generate prompt --id 20250809T130102+0000 --print-paths\n"
            )
        )
        self.app.add_typer(
            self.generate_app,
            name="generate",
            short_help="Generate artifacts",
        )
        self.generate_app.command(
            name="prompt",
            help="Generate prompt file and matching task folder",
            short_help="Generate prompt file",
        )(self._cmd_generate_prompt)
        self.app.command(
            name="generate-prompt",
            help="Generate prompt file and matching task folder",
            short_help="Generate prompt file",
        )(self._cmd_generate_prompt)

        self.app.add_typer(build_prompt_typer(self.container), name="prompt")
        self.app.add_typer(build_db_typer(), name="db")
        self.app.add_typer(build_import_typer(), name="import")

    
    def _callback(
        self,
        logfile: Path = typer.Option(LOG_FILE, help="Path to the log file"),
        db: Path | None = typer.Option(
            None,
            "--db",
            help="Path to SQLite DB (enables SQLite repo)",
        ),
    ) -> None:
        self.container.config.log_file.from_value(logfile)
        if db:
            self.container.config.db_path.from_value(db)

    def _cmd_hello_world(self) -> None:
        greeter = self.container.greeter()
        typer.echo(greeter.hello())

    def _cmd_divide(self, a: float, b: float) -> None:
        divider = self.container.divider()
        result = divider.divide(a, b)
        logging.getLogger(__name__).info("Divide %s by %s = %s", a, b, result)
        typer.echo(result)

    def _cmd_fail(self) -> None:
        raise FatalError("intentional failure")

    def _cmd_greet(self, name: str, excited: bool = False) -> None:
        greeter = self.container.greeter()
        typer.echo(greeter.greet(name, excited))

    def _cmd_set_project_path(
        self,
        path: Path | None = typer.Option(None, "--path", help="Set project root explicitly"),
        here: bool = typer.Option(
            False, "--here", help="Set project root to the current directory"
        ),
        unset: bool = typer.Option(False, "--unset", help="Clear the stored project root"),
        show: bool = typer.Option(False, "--show", help="Display the effective project root"),
    ) -> None:
        opts = [path is not None, here, unset, show]
        if sum(opts) != 1:
            typer.echo(
                "Specify exactly one of --path, --here, --unset, or --show",
                err=True,
            )
            raise typer.Exit(code=1)
        if show:
            root = resolve_project_root()
            typer.echo(str(root))
            raise typer.Exit(code=0)
        if unset:
            cfg = _config_file()
            if cfg.exists():
                cfg.unlink()
            typer.echo("Project path unset.")
            raise typer.Exit(code=0)
        if here:
            project_path = Path.cwd()
        else:
            assert path is not None
            project_path = Path(path).expanduser()
        if not project_path.is_dir():
            typer.echo(f"❌ Not a directory: {project_path}", err=True)
            raise typer.Exit(code=1)
        _write_config({"project_path": str(project_path)})
        typer.echo(f"Project path set to: {project_path}")
        raise typer.Exit(code=0)

    def _cmd_validate_today(
        self,
        timezone: str = typer.Option(
            "UTC", "--timezone", "-tz", help="Timezone for 'today' prompt folder (default: UTC)"
        ),
    ) -> None:
        project_root = resolve_project_root()
        prompts_dir = get_default_prompts_dir(project_root, timezone)
        typer.echo(
            f"Validating today's prompts under: {prompts_dir.resolve()} (timezone: {timezone})"
        )
        errors, passed, failed = validate_all(prompts_dir)
        if errors:
            typer.echo(f"[ERROR] Validation errors found in {failed} file(s):")
            for err in errors:
                typer.echo(f"  - {err}", err=True)
        elif passed == 0:
            typer.echo("No prompt files found.")
        else:
            typer.echo(
                f"[OK] All {passed} prompt files for today validated successfully!"
            )
        typer.echo(f"\nSummary: {passed} passed, {failed} failed.\n")
        if failed > 0:
            raise typer.Exit(code=1)
        raise typer.Exit(code=0)

    def _cmd_validate_all(
        self,
        prompts_dir: Path | None = typer.Option(
            None, help="Path to the prompts directory"
        ),
    ) -> None:
        project_root = resolve_project_root()
        if prompts_dir is None:
            prompts_dir = project_root / "prompts"
        else:
            prompts_dir = Path(prompts_dir)
        typer.echo(f"Validating ALL prompts under: {prompts_dir.resolve()}")
        errors, passed, failed = validate_all(prompts_dir)
        if errors:
            typer.echo(f"[ERROR] Validation errors found in {failed} file(s):")
            for err in errors:
                typer.echo(f"  - {err}", err=True)
        elif passed == 0:
            typer.echo("No prompt files found.")
        else:
            typer.echo("All prompt files are valid")
        typer.echo(f"\nSummary: {passed} passed, {failed} failed.\n")
        if failed > 0:
            raise typer.Exit(code=1)
        raise typer.Exit(code=0)

    def _cmd_harmonize_today(
        self,
        dry_run: bool = typer.Option(False, help="Show changes without saving"),
        timezone: str = typer.Option(
            "UTC", "--timezone", "-tz", help="Timezone for 'today' prompt folder (default: UTC)"
        ),
    ) -> None:
        project_root = resolve_project_root()
        prompts_dir = get_default_prompts_dir(project_root, timezone)
        typer.echo(
            f"Harmonizing TODAY's prompts under: {prompts_dir.resolve()} (timezone: {timezone})"
        )
        total_files = 0
        changed_files = 0
        for file_path in prompts_dir.rglob("*.md"):
            total_files += 1
            original = file_path.read_text(encoding="utf-8")
            processed, placeholders = preprocess_text(original)
            harmonized, actions = harmonize_text(processed)
            final = postprocess_text(harmonized, placeholders)
            if final != original:
                changed_files += 1
            if not dry_run and final != original:
                file_path.write_text(final, encoding="utf-8")
            if actions:
                for act in actions:
                    prefix = "Would " if dry_run else ""
                    typer.echo(f"[OK] {prefix}{act} in {file_path.name}")

        if total_files == 0:
            typer.echo("No prompt files found.")
        typer.echo(
            f"\nSummary: {changed_files if not dry_run else 0} file(s)"
            f"{' would be' if dry_run else ''} harmonized out of {total_files} checked.\n"
        )
        if dry_run:
            typer.echo("Dry run: No files were modified.\n")
        raise typer.Exit(code=0)

    def _cmd_harmonize_all(
        self,
        prompts_dir: Path | None = typer.Option(
            None, help="Path to the prompts directory"
        ),
        dry_run: bool = typer.Option(False, help="Show changes without saving"),
    ) -> None:
        project_root = resolve_project_root()
        if prompts_dir is None:
            prompts_dir = project_root / "prompts"
        else:
            prompts_dir = Path(prompts_dir)

        typer.echo(f"Harmonizing ALL prompts under: {prompts_dir.resolve()}")
        total_files = 0
        changed_files = 0
        for file_path in prompts_dir.rglob("*.md"):
            total_files += 1
            original = file_path.read_text(encoding="utf-8")
            processed, placeholders = preprocess_text(original)
            harmonized, actions = harmonize_text(processed)
            final = postprocess_text(harmonized, placeholders)
            if final != original:
                changed_files += 1
            if not dry_run and final != original:
                file_path.write_text(final, encoding="utf-8")
            if actions:
                for act in actions:
                    prefix = "Would " if dry_run else ""
                    typer.echo(f"[OK] {prefix}{act} in {file_path.name}")

        if total_files == 0:
            typer.echo("No prompt files found.")
        typer.echo(
            f"\nSummary: {changed_files if not dry_run else 0} file(s)"
            f"{' would be' if dry_run else ''} harmonized out of {total_files} checked.\n"
        )
        if dry_run:
            typer.echo("Dry run: No files were modified.\n")
        raise typer.Exit(code=0)

    def _cmd_generate_prompt(
        self,
        date: str | None = typer.Option(None, "--date", help="Override UTC date"),
        trace_id: str | None = typer.Option(
            None, "--id", help="Use specific ID; otherwise auto-generate"
        ),
        force: bool = typer.Option(False, "--force", help="Overwrite prompt file"),
        dry_run: bool = typer.Option(
            False, "--dry-run", help="Print actions; no writes"
        ),
        print_paths: bool = typer.Option(
            False, "--print-paths", help="Print abs paths for prompt + task"
        ),
    ) -> None:
        project_root = resolve_project_root()
        if date:
            try:
                dt = datetime.strptime(date, "%Y-%m-%d")
            except ValueError:
                typer.echo(
                    "❌ Invalid --date format. Expected YYYY-MM-DD (UTC).",
                    err=True,
                )
                raise typer.Exit(code=1) from None
        else:
            dt = datetime.now(ZoneInfo("UTC"))
        year = f"{dt.year:04}"
        month = f"{dt.month:02}"
        day = f"{dt.day:02}"

        if trace_id is None:
            tid = generate_trace_id("UTC")
        elif not is_valid_trace_id(trace_id):
            typer.echo(f"❌ Invalid trace id format: {trace_id}", err=True)
            raise typer.Exit(code=1)
        else:
            tid = trace_id

        prompts_dir = project_root / "prompts" / year / month / day
        tasks_dir = project_root / "tasks" / year / month / day
        prompt_file = prompts_dir / f"{tid}.md"
        task_folder = tasks_dir / tid

        # Skip collision check during dry-run
        if prompt_file.exists() and not force and not dry_run:
            typer.echo(
                f"❌ Prompt already exists: {prompt_file}. Use --force to overwrite.",
                err=True,
            )
            raise typer.Exit(code=1)

        logging.getLogger(__name__).info(
            "Generate prompt date=%s id=%s prompt=%s task=%s force=%s dry_run=%s",
            dt.date().isoformat(),
            tid,
            prompt_file,
            task_folder,
            force,
            dry_run,
        )

        # Read once so missing template surfaces even on --dry-run, and avoid re-reading later.
        template = importlib.resources.files("obk.templates").joinpath("prompt.xml").read_text(
            encoding="utf-8",
        )
        content = template.replace("__TRACE_ID__", tid)

        if dry_run:
            if print_paths:
                typer.echo(str(prompt_file.resolve()))
                typer.echo(str(task_folder.resolve()))
            else:
                typer.echo(f"Would create: {prompt_file.resolve()}")
                typer.echo(f"Would ensure: {task_folder.resolve()}")
            raise typer.Exit(code=0)

        # Real writes
        prompts_dir.mkdir(parents=True, exist_ok=True)
        task_folder.mkdir(parents=True, exist_ok=True)
        with prompt_file.open("w", encoding="utf-8", newline="\n") as fh:
            fh.write(content)

        # Output
        if print_paths:
            typer.echo(str(prompt_file.resolve()))
            typer.echo(str(task_folder.resolve()))
        else:
            typer.echo(f"✅ Created: {prompt_file.resolve()}")
            typer.echo(f"📂 Ensured: {task_folder.resolve()}")

        raise typer.Exit(code=0)

    def _cmd_trace_id(
        self,
        timezone: str = typer.Option(
            "UTC", "--timezone", "-tz", help="Timezone for trace ID generation (default: UTC)"
        ),
    ) -> None:
        typer.echo(generate_trace_id(timezone))

    def run(self, argv: list[str] | None = None) -> None:
        argv = argv or sys.argv[1:]
        try:
            cmd = typer.main.get_command(self.app)
            exit_code = cmd.main(args=argv, prog_name="obk", standalone_mode=False)
            if isinstance(exit_code, int):
                sys.exit(exit_code)
        except DivisionByZeroError as exc:
            logging.getLogger(__name__).exception("Division error")
            typer.echo(f"[ERROR] {exc}", err=True)
            sys.exit(2)


_cli = ObkCLI()
app = _cli.app

def main(argv: list[str] | None = None) -> None:
    """Entry point for ``python -m obk``."""
    argv = argv or sys.argv[1:]
    log_file = LOG_FILE
    if "--logfile" in argv:
        idx = argv.index("--logfile")
        if idx + 1 < len(argv):
            log_file = Path(argv[idx + 1])
    configure_logging(log_file)
    _install_global_excepthook()
    _cli.run(argv)


if __name__ == "__main__":
    main()




import xml.etree.ElementTree as ET
from importlib.resources import files
from pathlib import Path

import xmlschema

from .preprocess import preprocess_text

GLOBAL_ELEMENTS = {"gsl-label", "gsl-title", "gsl-description", "gsl-value"}


def get_schema_path(schema_name: str) -> Path:
    return Path(str(files("obk.xsd").joinpath(schema_name)))


def detect_prompt_type(file_path: Path) -> str:
    name = file_path.stem.lower()
    if "surgery" in name:
        return "surgery"
    return "gsl"

def validate_all(prompts_dir: Path) -> tuple[list[str], int, int]:
    """Validate all prompts under ``prompts_dir`` using mapped schemas."""
    errors: list[str] = []
    count_passed = 0
    count_failed = 0

    prompts_dir = prompts_dir.resolve()

    for file_path in prompts_dir.rglob("*.md"):
        text = file_path.read_text(encoding="utf-8")
        processed, _ = preprocess_text(text)
        schema_name = "prompt.xsd"
        schema_path = get_schema_path(schema_name)
        schema = xmlschema.XMLSchema(str(schema_path))
        try:
            file_errors = [
                err.reason or str(err) for err in schema.iter_errors(processed)
            ]
        except xmlschema.XMLSchemaException as exc:
            file_errors = [str(exc)]

        if not file_errors:
            root = ET.fromstring(processed)

            def _recurse(elem: ET.Element) -> None:
                seen = []
                for i, child in enumerate(list(elem)):
                    tag = child.tag
                    if tag in GLOBAL_ELEMENTS:
                        if tag in seen:
                            file_errors.append(
                                f"{file_path}: <{elem.tag}> has duplicate <{tag}>"
                            )
                        if any(c.tag not in GLOBAL_ELEMENTS for c in list(elem)[:i]):
                            file_errors.append(
                                f"{file_path}: <{tag}> must appear before other children in <{elem.tag}>"
                            )
                        seen.append(tag)
                    else:
                        if any(c.tag in GLOBAL_ELEMENTS for c in list(elem)[i + 1 :]):
                            file_errors.append(
                                f"{file_path}: global elements must precede other children in <{elem.tag}>"
                            )
                        break
                for child in elem:
                    _recurse(child)

            _recurse(root)

        if file_errors:
            for msg in file_errors:
                errors.append(f"{file_path}: {msg}")
            count_failed += 1
        else:
            count_passed += 1

    return errors, count_passed, count_failed



from __future__ import annotations

import typer


def build_db_typer() -> typer.Typer:
    """Build the 'db' sub-CLI. No side effects at import time."""
    app = typer.Typer(help="Database schema and migration commands")

    # Avoid B008: predefine parameter objects, then reference them in defaults.
    rules_opt = typer.Option(
        ...,
        "--rules",
        help="Path to schema/rules YAML",
    )
    preview_opt = typer.Option(
        False,
        "--preview",
        help="Preview compiled SQL only",
    )
    apply_opt = typer.Option(
        False,
        "--apply",
        help="Apply compiled SQL",
    )

    @app.command("compile-schema")
    def compile_schema(
        rules: str = rules_opt,
        preview: bool = preview_opt,
    ) -> None:
        # Stub implementation for tests
        typer.echo(f"[stub] compile-schema rules={rules} preview={preview}")

    @app.command("migrate")
    def migrate(
        apply: bool = apply_opt,
    ) -> None:
        # Stub implementation for tests
        typer.echo(f"[stub] migrate apply={apply}")

    return app


def build_import_typer() -> typer.Typer:
    """Build the 'import' sub-CLI. No side effects at import time."""
    app = typer.Typer(help="Structured imports")

    # Avoid B008: predefine parameter objects
    files_arg = typer.Argument(
        ...,
        help="One or more GSL files",
    )
    fail_on_opt = typer.Option(
        "schema,referential",
        "--fail-on",
        help="Comma list of failure gates",
    )

    @app.command("gsl")
    def import_gsl(
        files: list[str] = files_arg,
        fail_on: str = fail_on_opt,
    ) -> None:
        # Stub implementation for tests
        typer.echo(f"[stub] import gsl files={files} fail_on={fail_on}")

    return app



from __future__ import annotations

from collections.abc import Iterable, Sequence
from datetime import datetime
from typing import cast
from zoneinfo import ZoneInfo

import typer

from ...application.prompts.commands.add_prompt import AddPrompt
from ...application.prompts.queries.list_prompts import ListPrompts
from ...containers import Container


def _today_utc_str() -> str:
    # YYYY-MM-DD in UTC to satisfy domain expectations
    return datetime.now(ZoneInfo("UTC")).strftime("%Y-%m-%d")


def build_prompt_typer(container: Container | None = None) -> typer.Typer:
    """
    Returns a Typer app wired to send CQRS commands/queries through the mediator.
    No infra imports; strictly application-facing.
    """
    c = container or Container()
    app = typer.Typer(help="Manage prompts via CQRS handlers")

    @app.command("add")
    def add(
        id: str = typer.Argument(..., help="Prompt id"),
        type: str = typer.Argument(..., help="Prompt type"),
        date: str | None = typer.Option(None, "--date", "-d", help="UTC date YYYY-MM-DD"),
    ) -> None:
        d = date or _today_utc_str()
        c.mediator().send(AddPrompt(id=id, date_utc=d, type=type))
        typer.echo(f"Added\t{id}\t{d}\t{type}")

    @app.command("list")
    def list_() -> None:
        items = cast(
            Sequence[object],
            c.mediator().send(ListPrompts()),
        )# domain types, not enforced here
        for p in cast(Iterable[object], items):
            # duck-type: expects attributes id, date_utc, type
            _id = getattr(p, "id", "")
            _date = getattr(p, "date_utc", "")
            _type = getattr(p, "type", "")
            typer.echo(f"{_id}\t{_date}\t{_type}")

    return app




import os
import re
import subprocess
import sys
from pathlib import Path

import pytest
import typer

import obk.cli as cli

sys.path.insert(0, str(Path(__file__).resolve().parents[1] / "src"))


def test_hello_world():
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "hello-world"],
        capture_output=True,
        text=True,
        check=True,
        env=env,
    )
    assert result.stdout.strip() == "hello world"


def test_help():
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "hello-world", "-h"],
        capture_output=True,
        text=True,
        check=True,
        env=env,
    )
    assert "Print hello world" in result.stdout


def test_entrypoint_any_directory(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "hello-world"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.stdout.strip() == "hello world"


def test_divide_logs(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "divide", "4", "2"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.stdout.strip() == "2.0"
    log_file = tmp_path / "obk.log"
    assert log_file.exists()
    assert "Divide 4.0 by 2.0 = 2.0" in log_file.read_text()


def test_custom_logfile(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    log_file = tmp_path / "custom.log"
    result = subprocess.run(
        [sys.executable, "-m", "obk", "--logfile", str(log_file), "divide", "4", "2"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.stdout.strip() == "2.0"
    assert log_file.exists()


def test_divide_by_zero(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "divide", "1", "0"],
        capture_output=True,
        text=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.returncode == 2
    assert "cannot divide by zero" in result.stderr.lower()


def test_fail_exits_1(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "fail"],
        capture_output=True,
        text=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.returncode == 1
    assert "fatal" in result.stderr.lower()


def test_module_invocation(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "divide", "4", "2"],
        capture_output=True,
        text=True,
        cwd=tmp_path,
        env=env,
        check=True,
    )
    assert result.stdout.strip() == "2.0"


class MockGreeter:
    def hello(self) -> str:
        return "[mock] hi"


class MockDivider:
    def divide(self, a: float, b: float) -> float:
        return 42


def test_container_override_greeter(tmp_path, capsys):
    from obk.cli import ObkCLI
    from obk.containers import Container

    container = Container()
    container.greeter.override(MockGreeter())
    cli = ObkCLI(container=container, log_file=tmp_path / "log.log")
    cli.run(["hello-world"])
    captured = capsys.readouterr()
    assert captured.out.strip() == "[mock] hi"


def test_container_override_divider(tmp_path, capsys):
    from obk.cli import ObkCLI
    from obk.containers import Container

    container = Container()
    container.divider.override(MockDivider())
    cli = ObkCLI(container=container, log_file=tmp_path / "log.log")
    cli.run(["divide", "1", "3"])
    captured = capsys.readouterr()
    assert captured.out.strip() == "42"


def test_greet_excited(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "greet", "Ada", "--excited"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.stdout.strip() == "Hello, Ada!!!"


def test_greet_default(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "greet", "Ada"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.stdout.strip() == "Hello, Ada."


class MockGreeterGreet:
    def greet(self, name: str, excited: bool = False) -> str:
        return f"[mock greet] {name} {'!!!' if excited else ''}".strip()


def test_container_override_greeter_greet(tmp_path, capsys):
    from obk.cli import ObkCLI
    from obk.containers import Container

    container = Container()
    container.greeter.override(MockGreeterGreet())
    cli = ObkCLI(container=container, log_file=tmp_path / "log.log")
    cli.run(["greet", "Ada", "--excited"])
    captured = capsys.readouterr()
    assert captured.out.strip() == "[mock greet] Ada !!!"


def _write_prompt(file: Path, content: str) -> None:
    file.parent.mkdir(parents=True, exist_ok=True)
    file.write_text(content, encoding="utf-8")


def test_trace_id(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [sys.executable, "-m", "obk", "trace-id"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    assert re.match(r"\d{8}T\d{6}[+-]\d{4}", result.stdout.strip())



def test_validate_all_failure(tmp_path):
    bad = tmp_path / "prompts" / "bad.md"
    _write_prompt(bad, "<broken>")
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "obk",
            "validate-all",
            "--prompts-dir",
            str(tmp_path / "prompts"),
        ],
        capture_output=True,
        text=True,
        cwd=tmp_path,
        env=env,
    )
    assert result.returncode == 1
    assert "error" in result.stderr.lower() or result.stderr


def test_harmonize_all(tmp_path):
    prompt = tmp_path / "prompts" / "harm.md"
    _write_prompt(
        prompt,
        """<gsl-prompt id='20250731T000000+0000'>\n    <gsl-header>h</gsl-header>\n<gsl-block>\n    <gsl-purpose>p</gsl-purpose>\n</gsl-block>\n</gsl-prompt>\n""",
    )
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    env["OBK_PROJECT_PATH"] = str(tmp_path)
    subprocess.run(
        [
            sys.executable,
            "-m",
            "obk",
            "harmonize-all",
            "--prompts-dir",
            str(tmp_path / "prompts"),
        ],
        capture_output=True,
        text=True,
        cwd=tmp_path,
        env=env,
        check=True,
    )
    txt = prompt.read_text()
    assert txt.splitlines()[1].startswith("<gsl-header>")


def test_commands_any_directory(tmp_path):
    (tmp_path / "prompts").mkdir()
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    subprocess.run(
        [sys.executable, "-m", "obk", "trace-id"],
        capture_output=True,
        text=True,
        cwd=tmp_path,
        env=env,
        check=True,
    )
    assert (tmp_path / "prompts").exists()


def _extract_ymd(text: str) -> str:
    text = text.replace("\\", "/")
    match = re.search(r"prompts/(\d{4})/(\d{2})/(\d{2})", text)
    assert match
    return "".join(match.groups())



def test_today_commands_default_timezone(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    env["OBK_PROJECT_PATH"] = str(tmp_path)
    trace = subprocess.run(
        [sys.executable, "-m", "obk", "trace-id"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    validate = subprocess.run(
        [sys.executable, "-m", "obk", "validate-today"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    harmonize = subprocess.run(
        [sys.executable, "-m", "obk", "harmonize-today"],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    date = trace.stdout.strip()[:8]
    assert date == _extract_ymd(validate.stdout)
    assert date == _extract_ymd(harmonize.stdout)


def test_global_excepthook_system_exit(monkeypatch):
    called = {}

    def fake_hook(exc_type, exc, tb):
        called["ok"] = True

    monkeypatch.setattr(sys, "__excepthook__", fake_hook)
    cli._global_excepthook(SystemExit, SystemExit(), None)
    assert called.get("ok")


def test_validate_today_no_files(tmp_path, monkeypatch, capsys):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    prompts_dir = cli.get_default_prompts_dir(Path(tmp_path))
    prompts_dir.mkdir(parents=True)
    app = cli.ObkCLI(log_file=tmp_path / "log.log")
    with pytest.raises(typer.Exit) as exc:
        app._cmd_validate_today(timezone="UTC")
    captured = capsys.readouterr().out
    assert "No prompt files found." in captured
    assert exc.value.exit_code == 0


def test_validate_today_failure(tmp_path, monkeypatch, capsys):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    prompts_dir = cli.get_default_prompts_dir(Path(tmp_path))
    prompts_dir.mkdir(parents=True)
    (prompts_dir / "bad.md").write_text("<broken>", encoding="utf-8")
    app = cli.ObkCLI(log_file=tmp_path / "log.log")
    with pytest.raises(typer.Exit) as exc:
        app._cmd_validate_today(timezone="UTC")
    assert exc.value.exit_code == 1


VALID_PROMPT = (
    "<?xml version='1.0' encoding='UTF-8'?>\n"
    "<gsl-prompt id='20250731T000000+0000' type='feat'>\n"
    "  <gsl-header>h</gsl-header>\n"
    "  <gsl-block>\n"
    "    <gsl-purpose><gsl-title>Purpose</gsl-title><gsl-description>p</gsl-description></gsl-purpose>\n"
    "    <gsl-inputs><gsl-label>Input</gsl-label><gsl-value>i</gsl-value></gsl-inputs>\n"
    "    <gsl-outputs><gsl-label>Output</gsl-label><gsl-value>o</gsl-value></gsl-outputs>\n"
    "    <gsl-workflows><gsl-title>Steps</gsl-title><gsl-description>w</gsl-description></gsl-workflows>\n"
    "    <gsl-acceptance-tests>"
    "      <gsl-acceptance-test id='1'>"
    "        <gsl-performed-action>a</gsl-performed-action>"
    "        <gsl-expected-result>b</gsl-expected-result>"
    "      </gsl-acceptance-test>"
    "    </gsl-acceptance-tests>\n"
    "  </gsl-block>\n"
    "</gsl-prompt>\n"
)


def test_validate_all_no_files(tmp_path, capsys, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    empty = tmp_path / "prompts"
    empty.mkdir()
    app = cli.ObkCLI(log_file=tmp_path / "log.log")
    with pytest.raises(typer.Exit) as exc:
        app._cmd_validate_all(prompts_dir=empty)
    assert "No prompt files found." in capsys.readouterr().out
    assert exc.value.exit_code == 0


def test_validate_all_success(tmp_path, capsys, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    prompts = tmp_path / "prompts"
    prompts.mkdir()
    (prompts / "valid.md").write_text(VALID_PROMPT, encoding="utf-8")
    app = cli.ObkCLI(log_file=tmp_path / "log.log")
    with pytest.raises(typer.Exit) as exc:
        app._cmd_validate_all(prompts_dir=prompts)
    out = capsys.readouterr().out
    assert "All prompt files are valid" in out
    assert exc.value.exit_code == 0


def test_harmonize_all_no_files(tmp_path, capsys, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    empty = tmp_path / "prompts"
    empty.mkdir()
    app = cli.ObkCLI(log_file=tmp_path / "log.log")
    with pytest.raises(typer.Exit) as exc:
        app._cmd_harmonize_all(prompts_dir=empty)
    assert "No prompt files found." in capsys.readouterr().out
    assert exc.value.exit_code == 0


def test_today_commands_timezone_override(tmp_path):
    env = os.environ.copy()
    env["PYTHONPATH"] = str(Path(__file__).resolve().parents[1] / "src")
    env["OBK_PROJECT_PATH"] = str(tmp_path)
    tz = "America/New_York"
    trace = subprocess.run(
        [sys.executable, "-m", "obk", "trace-id", "--timezone", tz],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    validate = subprocess.run(
        [sys.executable, "-m", "obk", "validate-today", "--timezone", tz],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    harmonize = subprocess.run(
        [sys.executable, "-m", "obk", "harmonize-today", "--timezone", tz],
        capture_output=True,
        text=True,
        check=True,
        cwd=tmp_path,
        env=env,
    )
    date = trace.stdout.strip()[:8]
    assert date == _extract_ymd(validate.stdout)
    assert date == _extract_ymd(harmonize.stdout)



# src/obk/application/prompts/queries/list_prompts.py
from __future__ import annotations

from dataclasses import dataclass

from obk.application.prompts.ports import PromptRepository
from obk.domain.prompts import Prompt
from obk.infrastructure.mediator.core import Query


@dataclass(frozen=True)
class ListPrompts(Query):
    pass


def handle(_: ListPrompts, repo: PromptRepository) -> list[Prompt]:
    return repo.list()



from pathlib import Path

from obk.domain.prompts.models import Prompt
from obk.infrastructure.db.sqlite import SqlitePromptRepository, _connect


def test_pragmas_and_crud(tmp_path: Path) -> None:
    db = tmp_path / "session.db"
    repo = SqlitePromptRepository(db)
    repo.add(Prompt("1", "2025-08-10", "x"))
    assert [p.id for p in repo.list()] == ["1"]
    with _connect(db) as cx:
        assert cx.execute("PRAGMA foreign_keys").fetchone()[0] == 1
        assert str(cx.execute("PRAGMA journal_mode").fetchone()[0]).upper() == "WAL"



# src/obk/infrastructure/mediator/core.py
from __future__ import annotations

from collections.abc import Callable
from dataclasses import dataclass
from typing import Protocol, TypeVar

Req = TypeVar("Req", bound="Request")


class Behavior(Protocol):
    def wrap(self, next_: Callable[[Req], object]) -> Callable[[Req], object]: ...


@dataclass(frozen=True)
class Request: ...


@dataclass(frozen=True)
class Command(Request): ...


@dataclass(frozen=True)
class Query(Request): ...


Handler = Callable[[Request], object]


class Mediator:
    """Dispatch requests to handlers with optional behaviors."""

    def __init__(
        self,
        handlers: dict[type, Handler],
        behaviors: list[Behavior] | None = None,
    ) -> None:
        self._handlers = handlers
        self._behaviors = list(behaviors or [])

    def send(self, request: Request) -> object:
        req_type = type(request)
        if req_type not in self._handlers:
            raise KeyError(f"No handler registered for {req_type.__name__}")

        def invoke(req: Request) -> object:
            return self._handlers[req_type](req)

        fn: Callable[[Request], object] = invoke
        for behavior in reversed(self._behaviors):
            fn = behavior.wrap(fn)
        return fn(request)


class LoggingBehavior:
    def __init__(self, logger: Callable[[str], None]) -> None:
        self._log = logger

    def wrap(self, next_: Callable[[Request], object]) -> Callable[[Request], object]:
        def _wrapped(req: Request) -> object:
            self._log(f"[Mediator] Handling {type(req).__name__}")
            result = next_(req)
            self._log(f"[Mediator] Handled {type(req).__name__}")
            return result

        return _wrapped



from __future__ import annotations

import logging
from logging.handlers import RotatingFileHandler
from pathlib import Path


def test_log_rotation_rollover(tmp_path: Path) -> None:
    log_file = tmp_path / "app.log"
    logger = logging.getLogger("obk.test.rotation")
    logger.handlers.clear()
    logger.setLevel(logging.INFO)
    handler = RotatingFileHandler(log_file, maxBytes=128, backupCount=1, encoding="utf-8")
    handler.setFormatter(logging.Formatter("%(asctime)s %(levelname)s %(message)s"))
    logger.addHandler(handler)

    for i in range(200):
        logger.info("line %03d: %s", i, "x" * 40)

    logger.handlers.clear()
    handler.close()

    assert log_file.exists()
    rotated = log_file.with_suffix(".log.1")
    alt_rotated = log_file.with_name(log_file.name + ".1")
    assert rotated.exists() or alt_rotated.exists()



import os
import re
import subprocess
import sys
from pathlib import Path

import pytest

import obk.cli as cli

PYTHON = sys.executable
REPO_ROOT = Path(__file__).resolve().parents[1]
ENV = os.environ.copy()
ENV["PYTHONPATH"] = str(REPO_ROOT / "src")


def _run(args, cwd):
    return subprocess.run(
        [PYTHON, "-m", "obk", *args],
        capture_output=True,
        text=True,
        cwd=cwd,
        env=ENV,
    )


def _write_prompt(path: Path, content: str) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(content, encoding="utf-8")


def _set_project_root(tmp_path, monkeypatch):
    monkeypatch.setenv("OBK_PROJECT_PATH", str(tmp_path))
    return cli.get_default_prompts_dir(Path(tmp_path))


# ---------------- reusable XML -------------------------------------------
VALID_PROMPT = (
    "<?xml version='1.0' encoding='UTF-8'?>\n"
    "<gsl-prompt id='20250731T000000+0000'>\n"
    "  <gsl-header>h</gsl-header>\n"
    "  <gsl-block>\n"
    "    <gsl-purpose>p</gsl-purpose>\n"
    "    <gsl-inputs><gsl-value>i</gsl-value></gsl-inputs>\n"
    "    <gsl-outputs><gsl-value>o</gsl-value></gsl-outputs>\n"
    "    <gsl-workflows><gsl-value>w</gsl-value></gsl-workflows>\n"
    "    <gsl-tdd><gsl-test id='T1'>t</gsl-test></gsl-tdd>\n"
    "    <gsl-document-spec><gsl-value>d</gsl-value></gsl-document-spec>\n"
    "  </gsl-block>\n"
    "</gsl-prompt>\n"
)

BROKEN_PROMPT = "<broken>"


# ---------------- tests ---------------------------------------------------
# def test_validate_all(tmp_path, validate_all):
#     _write_prompt(tmp_path / "prompt1.md", VALID_PROMPT)
#
#     errors, passed, failed = validate_all(tmp_path)
#     assert not errors
#     assert passed == 1
#     assert failed == 0


def test_harmonize_all_and_dry_run(tmp_path):
    ENV["OBK_PROJECT_PATH"] = str(tmp_path)
    prompts = tmp_path / "prompts"
    prompt = prompts / "harm.md"
    content = (
        "<gsl-prompt id='20250731T000000+0000'>\n"
        "  <gsl-header>h</gsl-header>\n"
        "  <gsl-block>\n    <gsl-purpose>p</gsl-purpose>\n  </gsl-block>\n"
        "</gsl-prompt>\n"
    )
    _write_prompt(prompt, content)

    assert _run(["harmonize-all", "--prompts-dir", str(prompts)], cwd=tmp_path).returncode == 0
    txt = prompt.read_text()
    assert txt.splitlines()[1].startswith("<gsl-header>")

    _write_prompt(prompt, content)
    dry = _run(["harmonize-all", "--prompts-dir", str(prompts), "--dry-run"], cwd=tmp_path)
    assert dry.returncode == 0
    assert "Would" in dry.stdout
    assert prompt.read_text() == content


# def test_commands_any_directory_with_paths(tmp_path):
#     prompts = tmp_path / "prompts"
#     _write_prompt(prompts / "file.md", VALID_PROMPT)
#     sub = tmp_path / "subdir"
#     sub.mkdir()
#
#     assert _run(["validate-all", "--prompts-dir", str(prompts)], cwd=sub).returncode == 0
#     assert _run(["harmonize-all", "--prompts-dir", str(prompts)], cwd=sub).returncode == 0
#     r3 = _run(["trace-id"], cwd=sub)
#     assert r3.returncode == 0
#     assert re.match(r"\d{8}T\d{6}[+-]\d{4}", r3.stdout.strip())


def test_trace_id_timezones(tmp_path):
    assert re.match(r"\d{8}T\d{6}[+-]\d{4}", _run(
        ["trace-id", "--timezone", "America/New_York"], cwd=tmp_path).stdout.strip())

    bad = _run(["trace-id", "--timezone", "Invalid/Zone"], cwd=tmp_path)
    assert bad.returncode != 0
    text = (bad.stderr + bad.stdout).lower()
    assert "error" in text or "fatal" in text


# ------------- helpers for “today” tests ---------------------------------


# def test_validate_today(monkeypatch, capsys, tmp_path):
#     prompts_dir = _patch_repo_root(tmp_path, monkeypatch)
#     prompts_dir.mkdir(parents=True)
#
#     cli_obj = cli.ObkCLI()
#     with pytest.raises(SystemExit) as exc:
#         cli_obj.run(["validate-today"])
#     assert exc.value.code == 0
#
#     _write_prompt(prompts_dir / "bad.md", BROKEN_PROMPT)
#     with pytest.raises(SystemExit):
#         cli_obj.run(["validate-today"])
#
#     (prompts_dir / "bad.md").unlink()
#     _write_prompt(prompts_dir / "good.md", VALID_PROMPT)
#     with pytest.raises(SystemExit) as exc:
#         cli_obj.run(["validate-today"])
#     assert exc.value.code == 0


def test_harmonize_today(monkeypatch, capsys, tmp_path):
    prompts_dir = _set_project_root(tmp_path, monkeypatch)
    prompts_dir.mkdir(parents=True)

    file = prompts_dir / "harm.md"
    _write_prompt(
        file,
        "<gsl-prompt id='20250731T000000+0000'><gsl-header>h</gsl-header>"
        "<gsl-block><gsl-purpose>p</gsl-purpose></gsl-block></gsl-prompt>",
    )

    cli_obj = cli.ObkCLI()
    with pytest.raises(SystemExit):
        cli_obj.run(["harmonize-today"])

    _write_prompt(file, file.read_text())  # reset
    with pytest.raises(SystemExit):
        cli_obj.run(["harmonize-today", "--dry-run"])



# def test_autolocate_prompts_from_subdir(tmp_path):
#     prompts = tmp_path / "prompts"
#     _write_prompt(prompts / "good.md", VALID_PROMPT)
#     sub = tmp_path / "sub" / "inner"
#     sub.mkdir(parents=True)
#
#     assert _run(["validate-all"], cwd=sub).returncode == 0
#     assert _run(["harmonize-all"], cwd=sub).returncode == 0


def test_summary_no_prompts(monkeypatch, capsys, tmp_path):
    prompts_dir = _set_project_root(tmp_path, monkeypatch)
    prompts_dir.mkdir(parents=True)

    cli_obj = cli.ObkCLI()
    with pytest.raises(SystemExit):
        cli_obj.run(["harmonize-today"])
    captured = capsys.readouterr()
    assert "No prompt files found" in captured.out


def test_harmonize_all_dry_run_summary(tmp_path):
    ENV["OBK_PROJECT_PATH"] = str(tmp_path)
    prompts = tmp_path / "prompts"
    file = prompts / "harm.md"
    _write_prompt(
        file,
        "<gsl-prompt id='20250731T000000+0000'><gsl-header>h</gsl-header>"
        "<gsl-block><gsl-purpose>p</gsl-purpose></gsl-block></gsl-prompt>",
    )

    res = _run(["harmonize-all", "--prompts-dir", str(prompts), "--dry-run"], cwd=tmp_path)
    assert res.returncode == 0
    assert "Dry run" in res.stdout



from __future__ import annotations

# Thin shim to keep the public entry points/tests stable.
# - console script: obk = "obk.cli:main"
# - tests import: from obk.cli import ObkCLI, app, main
from .presentation.cli.main import (
    ObkCLI,
    _global_excepthook,  # legacy tests might import these
    app,
    get_default_prompts_dir,
    main,
)
from .project_path import resolve_project_root

__all__ = [
    "ObkCLI",
    "app",
    "main",
    "_global_excepthook",
    "get_default_prompts_dir",
    "resolve_project_root",
]




from pathlib import PureWindowsPath

from obk.trace_id import is_valid_trace_id


def test_windows_paths_format(tmp_path):
    # Simulate a known trace id and derived paths
    tid = "20250809T130102+0000"
    assert is_valid_trace_id(tid)
    win_prompt = PureWindowsPath(rf"prompts\\2025\\08\\09\\{tid}.md")
    win_task = PureWindowsPath(rf"tasks\\2025\\08\\09\\{tid}")
    # Pure path checks separator semantics without needing Windows
    assert "\\" in str(win_prompt)
    assert str(win_prompt).endswith(f"{tid}.md")
    assert "\\" in str(win_task)
    assert str(win_task).endswith(tid)
